﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Trees
{
    public static class Prufer
    {
        public static List<int[]> ReadFile(string path)
        {
            List<int[]> list = new List<int[]>();
            string[] line;
            int[] lineValues;
            using (StreamReader sr = new StreamReader(path))
            {
                while (!sr.EndOfStream)
                {
                    line = sr.ReadLine().Split(' ').ToArray();
                    lineValues = new int[2];
                    lineValues[0] = Convert.ToInt32(line[0]);
                    lineValues[1] = Convert.ToInt32(line[1]);
                    list.Add(lineValues);
                }
            }
            return list;
        }

        public static TreeNode CreateTree(List<int[]> Input)
        {
            int MaxValue = int.MinValue;
            int FirstChild = int.MinValue;
            int PairId = 0;
            for (int i = 0; i < Input.Count; i++)
            {
                if (Input[i][0] > MaxValue) 
                {
                    MaxValue = Input[i][0];
                    FirstChild = Input[i][1];
                    PairId = i;
                }
                if (Input[i][1] > MaxValue)
                {
                    MaxValue = Input[i][1];
                    FirstChild = Input[i][0];
                    PairId = i;
                }
            }
            Input.RemoveAt(PairId);
            TreeNode Tree = new TreeNode(MaxValue);
            Tree.AddChild(FirstChild);
            while (Input.Count > 0)
            {
                for (int i = 0; i < Input.Count; i++)
                {
                    TreeNode? Node = Tree.GetNodeRef(Input[i][0]);
                    if (Node == null) continue;
                    Node.AddChild(Input[i][1]);
                    PairId = i;
                    break;
                }
                Input.RemoveAt(PairId);
            }
            return Tree;
        }

        public static int[] FindMinChild(TreeNode Tree, int[] Values)
        {
            foreach (TreeNode child in Tree.Children)
            {
                if (child.Children.Count == 0 && child.Value < Values[1])
                {
                    Values[0] = Tree.Value;
                    Values[1] = child.Value;
                    return Values;
                }
                else if (child.Children.Count > 0)
                {
                    Values = FindMinChild(child, Values);
                }
            }
            return Values;
        }

        public static List<int> CreateCode(TreeNode Tree)
        {
            List<int> code = new List<int>();
            while (Tree.Children.Count >= 1)
            {
                int[] Values = new int[2];
                Values[1] = int.MaxValue;
                Values = FindMinChild(Tree, Values);
                Tree.RemoveChild(Values[1]);
                code.Add(Values[0]);
            }
            code.RemoveAt(code.Count -1);
            return code;
        }

        public static void PrintResult(List<int> Code)
        {
            using (StreamWriter FileWrite = new StreamWriter("Output.txt"))
            {
                FileWrite.WriteLine(String.Join(" ", Code));
            }
        }

        public static List<int[]> DecodePrufer(string path)
        {
            List<int[]> list = new List<int[]>();
            List<int> nodes = new List<int>();
            int[] values = new int[2];
            List<string> code;
            using (StreamReader sr = new StreamReader(path))
            {
                code = sr.ReadToEnd().Split(' ').ToList();
            }
            for (int i = 1; i <= code.Count + 2; i++)
            {
                nodes.Add(i);
            }
            while(nodes.Count > 2) 
            {
                values = new int[2];
                List<int> FreeNodes = nodes.Where(x => !code.Contains(x.ToString())).ToList();
                int node = FreeNodes.Min();
                nodes.Remove(node);
                values[0] = Convert.ToInt32(code[0]);
                code.Remove(code[0]);
                values[1] = node;
                list.Add(values);
            }
            values = new int[2];
            values[0] = nodes[0];
            values[1] = nodes[1];
            list.Add(values);
            return list;
        }
        public static void PrintTreeToFile(TreeNode node, string fileName)
        {
            using (StreamWriter writer = new StreamWriter(fileName))
            {
                PrintNode(node, writer, "");
            }
        }

        private static void PrintNode(TreeNode node, StreamWriter writer, string indent)
        {
            writer.WriteLine(indent + "├─ " + node.Value);
            for (int i = 0; i < node.Children.Count; i++)
            {
                if (i == node.Children.Count - 1)
                {
                    PrintNode(node.Children[i], writer, indent + "    ");
                }
                else
                {
                    writer.WriteLine(indent + "│");
                    PrintNode(node.Children[i], writer, indent + "│   ");
                }
            }
        }
    }

}
